@@ -3,7 +3,8 @@
 # calc.genoprob.R
 #
 # copyright (c) 2001, Karl W Broman, Johns Hopkins University
-# Sept, 2001; July, 2001; May, 2001; Apr, 2001; Feb, 2001
+# last modified Nov, 2001
+# first written Feb, 2001
 # Licensed under the GNU General Public License version 2 (June, 1991)
 # 
 # Part of the R/qtl package
@@ -22,7 +23,6 @@ calc.genoprob <-
 function(cross, step=0, off.end=0, error.prob=0,
          map.function=c("haldane","kosambi","c-f"))
 {
-
   # map function
   map.function <- match.arg(map.function)
   if(map.function=="kosambi") mf <- mf.k
@@ -30,14 +30,17 @@ function(cross, step=0, off.end=0, error.prob=0,
   else mf <- mf.h
  
   # don't let error.prob be exactly zero, just in case
-  if(error.prob < 1e-14) error.prob <- 1e-14
+  if(error.prob < 1e-50) error.prob <- 1e-50
 
   n.ind <- nind(cross)
   n.chr <- nchr(cross)
+  n.mar <- nmar(cross)
 
   # calculate genotype probabilities one chromosome at a time
   for(i in 1:n.chr) {
-
+    if(n.mar[i]==1) temp.offend <- max(c(off.end,5))
+    else temp.offend <- off.end
+    
     # which type of cross is this?
     if(class(cross)[1] == "f2") {
       one.map <- TRUE
@@ -74,7 +77,7 @@ function(cross, step=0, off.end=0, error.prob=0,
     # recombination fractions
     if(one.map) {
       # recombination fractions
-      map <- create.map(cross$geno[[i]]$map,step,off.end)
+      map <- create.map(cross$geno[[i]]$map,step,temp.offend)
       rf <- mf(diff(map))
       rf[rf < 1e-14] <- 1e-14
 
@@ -84,9 +87,10 @@ function(cross, step=0, off.end=0, error.prob=0,
       newgen[,colnames(gen)] <- gen
       newgen[is.na(newgen)] <- 0
       n.pos <- ncol(newgen)
+      marnames <- names(map)
     }
     else {
-      map <- create.map(cross$geno[[i]]$map,step,off.end)
+      map <- create.map(cross$geno[[i]]$map,step,temp.offend)
       rf <- mf(diff(map[1,]))
       rf[rf < 1e-14] <- 1e-14
       rf2 <- mf(diff(map[2,]))
@@ -98,9 +102,9 @@ function(cross, step=0, off.end=0, error.prob=0,
       newgen[,colnames(gen)] <- gen
       newgen[is.na(newgen)] <- 0
       n.pos <- ncol(newgen)
+      marnames <- colnames(map)
     }
 
-
     # call the C function
     if(one.map) {
       z <- .C(cfunc,
@@ -111,9 +115,6 @@ function(cross, step=0, off.end=0, error.prob=0,
               as.double(error.prob),     # 
               genoprob=as.double(rep(0,n.gen*n.ind*n.pos)),
               PACKAGE="qtl")
-
-      cross$geno[[i]]$prob <- array(z$genoprob,dim=c(n.ind,n.pos,n.gen))
-      dimnames(cross$geno[[i]]$prob) <- list(NULL, names(map), gen.names)
     }
     else {
       z <- .C(cfunc,
@@ -125,18 +126,18 @@ function(cross, step=0, off.end=0, error.prob=0,
               as.double(error.prob),     # 
               genoprob=as.double(rep(0,n.gen*n.ind*n.pos)),
               PACKAGE="qtl")
-
-      cross$geno[[i]]$prob <- array(z$genoprob,dim=c(n.ind,n.pos,n.gen))
-      dimnames(cross$geno[[i]]$prob) <-
-        list(NULL, colnames(map), gen.names)
     }
 
+    # re-arrange marginal probabilites
+    cross$geno[[i]]$prob <- array(z$genoprob,dim=c(n.ind,n.pos,n.gen))
+    dimnames(cross$geno[[i]]$prob) <- list(NULL, marnames, gen.names)
     # attribute set to the error.prob value used, for later
     #     reference, especially by calc.errorlod()
     attr(cross$geno[[i]]$prob,"error.prob") <- error.prob
     attr(cross$geno[[i]]$prob,"step") <- step
-    attr(cross$geno[[i]]$prob,"off.end") <- off.end
-  }
+    attr(cross$geno[[i]]$prob,"off.end") <- temp.offend
+    attr(cross$geno[[i]]$prob,"map.function") <- map.function
+  } # end loop over chromosomes
 
   cross
 }
