@@ -3,7 +3,10 @@
  * hmm_main.c
  *
  * copyright (c) 2001, Karl W Broman, Johns Hopkins University
- * Aug, 2001; Feb, 2001
+ *
+ * last modified Nov, 2001
+ * first written Feb, 2001
+ *
  * Licensed under the GNU General Public License version 2 (June, 1991)
  *
  * C functions for the R/qtl package
@@ -11,7 +14,7 @@
  * These functions are for the main HMM engine
  *
  * Contains: calc_genoprob, sim_geno, est_map, argmax_geno
- *           calc_errorlod, est_rf
+ *           calc_errorlod, est_rf, calc_pairprob
  *  
  **********************************************************************/
 
@@ -115,14 +118,24 @@ void calc_genoprob(int n_ind, int n_pos, int n_gen, int *geno,
 
     /* calculate genotype probabilities */
     for(j=0; j<n_pos; j++) {
+      s = Genoprob[0][j][i] = alpha[0][j] + beta[0][j];
+      for(v=1; v<n_gen; v++) {
+	Genoprob[v][j][i] = alpha[v][j] + beta[v][j];
+	s = addlog(s, Genoprob[v][j][i]);
+      }
+      for(v=0; v<n_gen; v++) 
+	Genoprob[v][j][i] = exp(Genoprob[v][j][i] - s);
+    } 
+
+    /* the following is the old version */
+    /*    for(j=0; j<n_pos; j++) {
       s = 0.0;
       for(v=0; v<n_gen; v++) 
 	s += (Genoprob[v][j][i] = exp(alpha[v][j] + beta[v][j]));
 
       for(v=0; v<n_gen; v++) 
 	Genoprob[v][j][i] /= s;
-
-    }
+	} */
 
 
   } /* loop over individuals */
@@ -185,6 +198,8 @@ void sim_geno(int n_ind, int n_pos, int n_gen, int n_draws,
   
   /* allocate space for beta and 
      reorganize geno and draws */
+  /* Geno indexed as Geno[pos][ind] */
+  /* Draws indexed as Draws[rep][pos][ind] */
   reorg_geno(n_ind, n_pos, geno, &Geno);
   reorg_draws(n_ind, n_pos, n_draws, draws, &Draws);
   allocate_alpha(n_pos, n_gen, &beta);
@@ -197,8 +212,7 @@ void sim_geno(int n_ind, int n_pos, int n_gen, int n_draws,
 
     /* do backward equations */
     /* initialize beta */
-    for(v=0; v<n_gen; v++) 
-      beta[v][n_pos-1] = 0.0;
+    for(v=0; v<n_gen; v++) beta[v][n_pos-1] = 0.0;
 
     /* backward equations */
     for(j=n_pos-2; j>=0; j--) {
@@ -211,33 +225,33 @@ void sim_geno(int n_ind, int n_pos, int n_gen, int n_draws,
 	  beta[v][j] = addlog(beta[v][j], beta[v2][j+1] + 
 			      stepf(v+1,v2+1,rf[j],rf2[j]) +
 			      emitf(Geno[j+1][i],v2+1,error_prob));
-
       }
     }
 
-    for(k=0; k<n_draws; k++) { /* k = simulate replicate */
+    for(k=0; k<n_draws; k++) { /* k = simulation replicate */
 
       /* first draw */
       /* calculate probs */
-      for(v=0, s=0; v<n_gen; v++) 
-	s += (probs[v] = exp(initf(v+1) + emitf(Geno[0][i], v+1, error_prob) +
-			     beta[v][0]));
-      for(v=0; v<n_gen; v++)
-	probs[v] /= s;
+      s = (probs[0] = initf(1)+emitf(Geno[0][i],1,error_prob)+beta[0][0]);
+      for(v=1; v<n_gen; v++) {
+	probs[v] = initf(v+1) + emitf(Geno[0][i], v+1, error_prob) +
+	  beta[v][0];
+	s = addlog(s, probs[v]);
+      }
+      for(v=0; v<n_gen; v++) probs[v] = exp(probs[v] - s);
 
-      /* make draw */
+      /* make draw: returns a value from {1, 2, ..., n_gen} */
       curstate = Draws[k][0][i] = sample_int(n_gen, probs);
       
       /* move along chromosome */
       for(j=1; j<n_pos; j++) {
 	/* calculate probs */
-	for(v=0, s=0; v<n_gen; v++) 
+	for(v=0; v<n_gen; v++) 
 	  probs[v] = exp(stepf(curstate,v+1,rf[j-1],rf2[j-1]) +
 			 emitf(Geno[j][i],v+1,error_prob) +
 			 beta[v][j] - beta[curstate-1][j-1]);
 	/* make draw */
 	curstate = Draws[k][j][i] = sample_int(n_gen, probs);
-
       }
 
     } /* loop over replicates */
@@ -303,7 +317,7 @@ void est_map(int n_ind, int n_mar, int n_gen, int *geno, double *rf,
 	     double stepf(int, int, double, double), 
 	     double nrecf1(int, int), double nrecf2(int, int), 
 	     double *loglik, int maxit, double tol, int sexsp, 
-	     int prnt)
+	     int trace)
 {
   int i, j, j2, v, v2, it, flag=0, **Geno;
   double s, **alpha, **beta, **gamma, *cur_rf, *cur_rf2;
@@ -317,11 +331,11 @@ void est_map(int n_ind, int n_mar, int n_gen, int *geno, double *rf,
   allocate_double(n_mar-1, &cur_rf);
   allocate_double(n_mar-1, &cur_rf2);
 
-  if(prnt) {
+  if(trace) {
     /* print initial estimates */
     Rprintf("      "); 
     for(j=0; j<n_mar-1; j++) Rprintf("%.3lf ", rf[j]);
-    Rprintf("\n");
+    Rprintf("\n"); 
   }
 
   /* begin EM algorithm */
@@ -394,20 +408,20 @@ void est_map(int n_ind, int n_mar, int n_gen, int *geno, double *rf,
     /* rescale */
     for(j=0; j<n_mar-1; j++) {
       rf[j] /= (double)n_ind;
-      if(rf[j] < tol) rf[j] = tol;
-      else if(rf[j] > 0.5-tol) rf[j] = 0.5-tol;
+      if(rf[j] < tol/100.0) rf[j] = tol/100.0;
+      else if(rf[j] > 0.5-tol/100.0) rf[j] = 0.5-tol/100.0;
       
       if(sexsp) {
 	rf2[j] /= (double)n_ind;
-	if(rf2[j] < tol) rf2[j] = tol;
-	else if(rf2[j] > 0.5-tol) rf2[j] = 0.5-tol;
+	if(rf2[j] < tol/100.0) rf2[j] = tol/100.0;
+	else if(rf2[j] > 0.5-tol) rf2[j] = 0.5-tol/100.0;
       }
     }
 
     /* check convergence */
     for(j=0, flag=0; j<n_mar-1; j++) {
-      if(fabs(rf[j] - cur_rf[j]) > tol || 
-	 (sexsp && fabs(rf2[j] - cur_rf2[j]) > tol)) {
+      if(fabs(rf[j] - cur_rf[j]) > tol*(cur_rf[j]+tol*100.0) || 
+	 (sexsp && fabs(rf2[j] - cur_rf2[j]) > tol*(cur_rf2[j]+tol*100.0))) {
 	flag = 1; 
 	break;
       }
@@ -443,7 +457,7 @@ void est_map(int n_ind, int n_mar, int n_gen, int *geno, double *rf,
     *loglik += curloglik;
   }
 
-  if(prnt) {
+  if(trace) {
     /* print final estimates */
     Rprintf(" %4d ", it+1);
     for(j=0; j<n_mar-1; j++) Rprintf("%.3lf ", rf[j]);
@@ -504,7 +518,7 @@ void argmax_geno(int n_ind, int n_pos, int n_gen, int *geno,
 		 double stepf(int, int, double, double)) 
 {
   int i, j, v, v2;
-  double s, t, *gamma, *tempgamma;
+  double s, t, *gamma, *tempgamma, *tempgamma2;
   int **Geno, **Argmax, **traceback;
   
   /* Read R's random seed */
@@ -519,6 +533,7 @@ void argmax_geno(int n_ind, int n_pos, int n_gen, int *geno,
   allocate_imatrix(n_pos, n_gen, &traceback);
   allocate_double(n_gen, &gamma);
   allocate_double(n_gen, &tempgamma);
+  allocate_double(n_gen, &tempgamma2);
 
   for(i=0; i<n_ind; i++) { /* i = individual */
 
@@ -539,9 +554,9 @@ void argmax_geno(int n_ind, int n_pos, int n_gen, int *geno,
 	      traceback[j][v] = v2;
 	    }
 	  }
-	  gamma[v] = tempgamma[v] + emitf(Geno[j+1][i], v+1, error_prob);
+	  tempgamma2[v] = tempgamma[v] + emitf(Geno[j+1][i], v+1, error_prob);
 	}
-
+	for(v=0; v<n_gen; v++) gamma[v] = tempgamma2[v];
       }
     
       /* finish off viterbi and then traceback to get most 
@@ -571,8 +586,7 @@ void argmax_geno(int n_ind, int n_pos, int n_gen, int *geno,
     }
     
     /* code genotypes as 1, 2, ... */
-    for(j=0; j<n_pos; j++) 
-      Argmax[j][i]++;
+    for(j=0; j<n_pos; j++) Argmax[j][i]++;
     
   } /* loop over individuals */
   
@@ -703,7 +717,7 @@ void est_rf(int n_ind, int n_mar, int *geno, double *rf,
 
 	  next_rf /= (double) n_mei;
 	  
-	  if(fabs(next_rf - cur_rf) < tol) { 
+	  if(fabs(next_rf - cur_rf) < tol*(cur_rf+tol*100.0)) { 
 	    flag = 1;
 	    break;
 	  }
@@ -732,4 +746,156 @@ void est_rf(int n_ind, int n_mar, int *geno, double *rf,
   }
 }
 
+/**********************************************************************
+ * 
+ * calc_pairprob
+ *
+ * This function uses the hidden Markov model technology to calculate 
+ * the joint genotype probabilities for all pairs of putative QTLs.
+ * This assumes data on a single chromosome
+ *
+ * n_ind        Number of individuals
+ *
+ * n_pos        Number of markers (or really positions at which to 
+ *              calculate the genotype probabilities)
+ *
+ * n_gen        Number of different genotypes
+ *  
+ * geno         Genotype data, as a single vector storing the matrix 
+ *              by columns, with each column corresponding to a marker
+ *
+ * rf           Recombination fractions
+ *
+ * rf2          A second set of recombination fractions, in case of
+ *              sex-specific maps (may be ignored)
+ *
+ * error_prob   Genotyping error probability
+ *
+ * genoprob     Genotype probabilities (the output); a single vector, 
+ *              of length n_ind x n_pos x n_gen, stored by columns 
+ *              (ind moves fastest, then mar, then genotype
+ *
+ * pairprob     Joint genotype probabilities for pairs of positions.
+ *              A single vector of length n_ind x n_pos x (n_pos-1)/2 x
+ *              n_gen^2.  We only calculate probabilities for 
+ *              pairs (i,j) with i < j.
+ *
+ * initf        Function returning log Pr(g_i)
+ *
+ * emitf        Function returning log Pr(O_i | g_i)
+ * 
+ * stepf        Function returning log Pr(g_2 | g_1)
+ *
+ **********************************************************************/
+
+/* Note: true genotypes coded as 1, 2, ...
+   but in the alpha's and beta's, we use 0, 1, ... */
+
+void calc_pairprob(int n_ind, int n_pos, int n_gen, int *geno, 
+		   double *rf, double *rf2, 
+		   double error_prob, double *genoprob, 
+		   double *pairprob, 
+		   double initf(int), 
+		   double emitf(int, int, double),
+		   double stepf(int, int, double, double)) 
+{
+  int i, j, j2, v, v2, v3;
+  double s=0.0, **alpha, **beta;
+  int **Geno;
+  double ***Genoprob, *****Pairprob;
+  
+  /* n_pos must be at least 2, or there are no pairs! */
+  if(n_pos < 2) error("n_pos must be > 1 in calc_pairprob");
+
+  /* allocate space for alpha and beta and 
+     reorganize geno, genoprob, and pairprob */
+  reorg_geno(n_ind, n_pos, geno, &Geno);
+  reorg_genoprob(n_ind, n_pos, n_gen, genoprob, &Genoprob);
+  reorg_pairprob(n_ind, n_pos, n_gen, pairprob, &Pairprob);
+  allocate_alpha(n_pos, n_gen, &alpha);
+  allocate_alpha(n_pos, n_gen, &beta);
+
+  for(i=0; i<n_ind; i++) { /* i = individual */
+
+    /* initialize alpha and beta */
+    for(v=0; v<n_gen; v++) {
+      alpha[v][0] = initf(v+1) + emitf(Geno[0][i], v+1, error_prob);
+      beta[v][n_pos-1] = 0.0;
+    }
+
+    /* forward-backward equations */
+    for(j=1,j2=n_pos-2; j<n_pos; j++, j2--) {
+      
+      for(v=0; v<n_gen; v++) {
+	alpha[v][j] = alpha[0][j-1] + stepf(1, v+1, rf[j-1], rf2[j-1]);
+	
+	beta[v][j2] = beta[0][j2+1] + stepf(v+1,1,rf[j2], rf2[j2]) + 
+	  emitf(Geno[j2+1][i],1,error_prob);
+
+	for(v2=1; v2<n_gen; v2++) {
+	  alpha[v][j] = addlog(alpha[v][j], alpha[v2][j-1] + 
+			       stepf(v2+1,v+1,rf[j-1],rf2[j-1]));
+	  beta[v][j2] = addlog(beta[v][j2], beta[v2][j2+1] + 
+			       stepf(v+1,v2+1,rf[j2],rf2[j2]) +
+			       emitf(Geno[j2+1][i],v2+1,error_prob));
+	}
+
+	alpha[v][j] += emitf(Geno[j][i],v+1,error_prob);
+      }
+    }
+
+    /* calculate genotype probabilities */
+    for(j=0; j<n_pos; j++) {
+      s = Genoprob[0][j][i] = alpha[0][j] + beta[0][j];
+      for(v=1; v<n_gen; v++) {
+	Genoprob[v][j][i] = alpha[v][j] + beta[v][j];
+	s = addlog(s, Genoprob[v][j][i]);
+      }
+      for(v=0; v<n_gen; v++) 
+	Genoprob[v][j][i] = exp(Genoprob[v][j][i] - s);
+    }
+
+    /* calculate Pr(G[j], G[j+1] | marker data) for i = 1...n_pos-1 */
+    for(j=0; j<n_pos-1; j++) {
+      for(v=0; v<n_gen; v++) {
+	for(v2=0; v2<n_gen; v2++) {
+	  Pairprob[v][v2][j][j+1][i] = alpha[v][j] + beta[v2][j+1] +
+	    stepf(v+1,v2+1,rf[j],rf2[j]) + 
+	    emitf(Geno[j+1][i],v2+1,error_prob);
+	  if(v==0 && v2==0) s=Pairprob[v][v2][j][j+1][i];
+	  else s = addlog(s,Pairprob[v][v2][j][j+1][i]);
+	}
+      }
+      /* scale to sum to 1 */
+      for(v=0; v<n_gen; v++) 
+	for(v2=0; v2<n_gen; v2++) 
+	  Pairprob[v][v2][j][j+1][i] = 
+	    exp(Pairprob[v][v2][j][j+1][i] - s);
+    } 
+
+    /* now calculate Pr(G[i], G[j] | marker data) for j > i+1 */
+    for(j=0; j<n_pos-2; j++) {
+      for(j2=j+2; j2<n_pos; j2++) {
+
+	for(v=0; v<n_gen; v++) { /* genotype at pos'n j */
+	  for(v2=0; v2<n_gen; v2++) { /* genotype at pos'n j2 */
+
+	    Pairprob[v][v2][j][j2][i] = 0.0;
+
+	    for(v3=0; v3<n_gen; v3++) { /* genotype at pos'n j2-1 */
+	      s = Genoprob[v3][j2-1][i];
+	      if(fabs(s) > TOL) /* avoid 0/0 */
+		Pairprob[v][v2][j][j2][i] += Pairprob[v][v3][j][j2-1][i]*
+		  Pairprob[v3][v2][j2-1][j2][i]/s;
+	    }
+		      
+	  }
+	} /* end loops over genotypes */
+
+      }
+    } /* end loops over pairs of positions */
+	    
+  } /* end loop over individuals */
+}
+
 /* end of hmm_main.c */
